#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'aprs4r/APRS4RLogger'

require 'aprs4r/GatewayPluginConfiguration'
require 'aprs4r/Plugin'
require 'aprs4r/SocketManager'


module APRS4R

  class GatewayPlugin < Plugin

    @logger = APRS4RLogger.get_logger( "GatewayPlugin")


    def initialize( configuration)
      logger.info( "initialize( configuration)")

      super( configuration)

      @inDevice = configuration.inDevice
      @outDevice = configuration.outDevice

      return
    end


    def registerListener
      logger.info( "registerListener")

      if @enable
        SocketManager.addListener( @inDevice, self)
      end

      return
    end


    def unregisterListener
      logger.info( "registerListener")

      if @enable
        SocketManager.removeListener( @inDevice, self)
      end

      return
    end


    def start
      logger.info( "start")

      if ! @enable 
        return
      end

      registerListener

      return
    end

    
    def receiveAPRSMessage( name, message)
      logger.info( "receiveAPRSMessage( name, message)")
      
      if message.nil?
        logger.warn( "message: NIL")
        return
      end
      
      # NOGATE check
      path = message.path
      if path.nil? || path.include?( "NOGATE") || path.include?( "RFONLY")
        return
      end
      
      logger.log( "gating from #{name} to #{@outDevice}, message: #{message}")
      SocketManager.sendAPRSMessage( @outDevice, message)
      
      return
    end

  end

end
