#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'socket'

require 'aprs4r/APRS4RLogger'

require 'aprs4r/ISServerPluginConfiguration'
require 'aprs4r/ISServerSocket'

require 'aprs4r/Plugin'


module APRS4R

  class ISServerPlugin < Plugin

    @logger = APRS4RLogger.get_logger( "ISServerPlugin")

    
    def initialize( configuration)
      logger.info( "initialize( configuration)")

      Thread.abort_on_exception = true
      
      super( configuration)

      @port = configuration.port

      @inDevices = Array.new
      @inDevices = configuration.inDevices if configuration.inDevices
      
      @outDevices = Array.new
      @outDevices = configuration.outDevices if configuration.outDevices

      @serverSocket = ISServerSocket.new( @port)
      @clientSockets = Array.new

      return
    end

    
    def start 
      logger.info( "start")

      if ! @enable
        return
      end

      Thread.new { runServerThread } 

      @inDevices.each{ |device|
        registerListener( device)
      }

      return
    end

    
    def registerListener( device)
      logger.info( "registerListener")

      if @enable
        SocketManager.addListener( device, self)
      end

      return
    end


    def unregisterListener( device)
      logger.info( "registerListener")

      if @enable
        SocketManager.removeListener( device, self)
      end

      return
    end


    def runServerThread
      logger.info( "run")

      while true 
        clientSocket = @serverSocket.accept

        logger.log( "clientSocket: #{clientSocket}")

        if clientSocket 
          Thread.new { runClientThread( clientSocket) } 
          @clientSockets << clientSocket
        end

      end
    end


    def runClientThread( clientSocket)
      logger.info( "runClientThead( clientSocket)")

      run = true 

      while run
        begin
          message = clientSocket.readAPRSMessage
          logger.log( "recv local is message: #{message}")
          
          @outDevices.each{ |device|
            SocketManager.sendAPRSMessage( device, message)
          }

        rescue Exception => ex
          logger.warn( "runClientThread::ex: " + ex)
          @clientSockets.delete( clientSocket)
          run = false
        end
      end

      return
    end

    
    def receiveAPRSMessage( name, message)
      logger.info( "receiveAPRSMessage( name, message)")

      if message.nil? 
        return
      end

      @clientSockets.each{ |clientSocket|
        
        begin
          logger.log( "send local is message: #{message}")
          clientSocket.writeAPRSMessage( message)
        rescue Exception => ex
          logger.warn( "receiveAPRSMessage::ex: " + ex)
          @clientSockets.delete( clientSocket)
        end

      }
      
      return
    end

  end

end
